@@ -13,7 +13,7 @@ import numpy
 from tslearn.metrics import cdist_gak, cdist_dtw, cdist_soft_dtw, dtw
 from tslearn.barycenters import EuclideanBarycenter, DTWBarycenterAveraging, SoftDTWBarycenter
 from tslearn.preprocessing import TimeSeriesScalerMeanVariance
-from tslearn.utils import to_time_series_dataset, check_equal_size, ts_size
+from tslearn.utils import to_time_series_dataset, to_time_series, ts_size
 from tslearn.cycc import cdist_normalized_cc, y_shifted_sbd_vec
 
 
@@ -68,7 +68,7 @@ def _compute_inertia(distances, assignments, squared=True):
         return numpy.sum(distances[numpy.arange(n_ts), assignments]) / n_ts
 
 
-def silhouette_score(X, labels, metric=None, sample_size=None,
+def silhouette_score(X, labels, metric=None, sample_size=None, metric_params=None,
                      random_state=None, **kwds):
     """Compute the mean Silhouette Coefficient of all samples (cf.  [1]_ and  [2]_).
 
@@ -84,12 +84,16 @@ def silhouette_score(X, labels, metric=None, sample_size=None,
          Predicted labels for each time series.
     metric : string, or callable
         The metric to use when calculating distance between time series.
-        Should be a metric from :ref:`tslearn.metrics <mod-metrics>` If X is the distance
-        array itself, use ``metric="precomputed"``.
+        Should be one of {'dtw', 'softdtw', 'euclidean'} or a callable distance
+        function.
+        If X is the distance array itself, use ``metric="precomputed"``.
     sample_size : int or None
         The size of the sample to use when computing the Silhouette Coefficient
         on a random subset of the data.
         If ``sample_size is None``, no sampling is used.
+    metric_params : dict or None
+        Parameter values for the chosen metric. Value associated to the `"gamma_sdtw"` key corresponds to the gamma
+        parameter in Soft-DTW.
     random_state : int, RandomState instance or None, optional (default=None)
         The generator used to randomly select a subset of samples.  If int,
         random_state is the seed used by the random number generator; If
@@ -113,28 +117,43 @@ def silhouette_score(X, labels, metric=None, sample_size=None,
     Examples
     --------
     >>> from tslearn.generators import random_walks
-    >>> from tslearn.metrics import soft_dtw
+    >>> from tslearn.metrics import cdist_dtw
     >>> X = random_walks(n_ts=50, sz=32, d=1)
     >>> labels = numpy.random.randint(2, size=50)
-    >>> s_sc = silhouette_score(X, labels, metric=dtw)
-    >>> s_sc2 = silhouette_score(X, labels, metric=soft_dtw)
+    >>> s_sc = silhouette_score(X, labels, metric="dtw")
+    >>> s_sc2 = silhouette_score(X, labels, metric="softdtw")
+    >>> s_sc2b = silhouette_score(X, labels, metric="softdtw", metric_params={"gamma_sdtw": 2.})
     >>> s_sc3 = silhouette_score(cdist_dtw(X), labels, metric="precomputed")
     """
+    sklearn_metric = None
+    if metric_params is None:
+        metric_params = {}
     if metric == "precomputed":
         sklearn_X = X
-        sklearn_metric = metric
+    elif metric == "dtw":
+        sklearn_X = cdist_dtw(X)
+    elif metric == "softdtw":
+        gamma = metric_params.get("gamma_sdtw", None)
+        if gamma is not None:
+            sklearn_X = cdist_soft_dtw(X, gamma=gamma)
+        else:
+            sklearn_X = cdist_soft_dtw(X)
+    elif metric == "euclidean":
+        sklearn_X = cdist(X, X, metric="euclidean")
     else:
         X_ = to_time_series_dataset(X)
         n, sz, d = X_.shape
         sklearn_X = X_.reshape((n, -1))
         if metric is None:
             metric = dtw
-        sklearn_metric = lambda x, y: metric(x.reshape((sz, d)), y.reshape((sz, d)))
+        sklearn_metric = lambda x, y: metric(to_time_series(x.reshape((sz, d)), remove_nans=True),
+                                             to_time_series(y.reshape((sz, d)), remove_nans=True))
     return sklearn_silhouette_score(X=sklearn_X,
                                     labels=labels,
-                                    metric=sklearn_metric,
+                                    metric="precomputed" if sklearn_metric is None else sklearn_metric,
                                     sample_size=sample_size,
-                                    random_state=random_state, **kwds)
+                                    random_state=random_state,
+                                    **kwds)
 
 
 class GlobalAlignmentKernelKMeans(BaseEstimator, ClusterMixin):
@@ -423,6 +442,9 @@ class TimeSeriesKMeans(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClust
     True
     >>> TimeSeriesKMeans(n_clusters=101, verbose=False, random_state=0).fit(X).X_fit_ is None
     True
+    >>> X_bis = to_time_series_dataset([[1, 2, 3, 4], [1, 2, 3], [2, 5, 6, 7, 8, 9]])
+    >>> km = TimeSeriesKMeans(n_clusters=2, verbose=False, max_iter=5, metric="dtw", random_state=0).fit(X_bis)
+    >>> km_bis = TimeSeriesKMeans(n_clusters=2, verbose=False, max_iter=5, metric="softdtw", random_state=0).fit(X_bis)
     """
 
     def __init__(self, n_clusters=3, max_iter=50, tol=1e-6, n_init=1, metric="euclidean", max_iter_barycenter=100,
@@ -499,7 +521,8 @@ class TimeSeriesKMeans(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClust
                                                                   verbose=False).fit(X[self.labels_ == k])
             elif self.metric == "softdtw":
                 self.cluster_centers_[k] = SoftDTWBarycenter(max_iter=self.max_iter_barycenter,
-                                                             gamma=self.gamma_sdtw).fit(X[self.labels_ == k])
+                                                             gamma=self.gamma_sdtw,
+                                                             init=self.cluster_centers_[k]).fit(X[self.labels_ == k])
             else:
                 self.cluster_centers_[k] = EuclideanBarycenter().fit(X[self.labels_ == k])
 
