@@ -325,4 +325,8 @@ def ts_size(ts):
     sz = ts_.shape[0]
     while not numpy.any(numpy.isfinite(ts_[sz - 1])):
         sz -= 1
-    return sz
\ No newline at end of file
+    return sz
+
+
+def ts_zeros(sz, d=1):
+    return numpy.zeros((sz, d))
